package ai.guiji.live.app;

import android.app.Application;
import android.text.TextUtils;
import android.util.Log;

import com.lzy.okgo.OkGo;
import com.lzy.okgo.cache.CacheEntity;
import com.lzy.okgo.cache.CacheMode;
import com.lzy.okgo.https.HttpsUtils;
import com.lzy.okgo.interceptor.HttpLoggingInterceptor;
import com.lzy.okgo.model.HttpHeaders;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;

import ai.guiji.live.app.pojo.MediaInfo;
import ai.guiji.live.app.util.DeviceUtils;
import ai.guiji.live.app.util.SystemUtils;
import okhttp3.OkHttpClient;

public class App extends Application {

    private static App mApp;

    @Override
    public void onCreate() {
        super.onCreate();
        mApp = this;
        initOkGo();
    }

    public static App getApp() {
        return mApp;
    }

    // 临时数据存储
    private List<MediaInfo> medias = new ArrayList<>();

    public List<MediaInfo> getMedias() {
        return medias;
    }

    public void setMedias(List<MediaInfo> medias) {
        this.medias = medias;
    }

    public String mqttUsername;
    public String mqttPassword;

    public void initOkGo() {
        try {
            Log.d("OkGoLive", "initOkGo: ");
            String uuid = DeviceUtils.getUUID(mApp);
            HttpHeaders headers = new HttpHeaders();
            headers.put("deviceType", "2");
            headers.put("deviceID", uuid);
            headers.put("version", SystemUtils.getVersionName(mApp));
            headers.put("skipVersion", "1");
            headers.put("flavor", "guiji");
            headers.put("source", "sdk");

            OkHttpClient.Builder builder = new OkHttpClient.Builder();
            //log相关
            HttpLoggingInterceptor loggingInterceptor = new HttpLoggingInterceptor("OkGoLive");
            if (BuildConfig.DEBUG) {
                loggingInterceptor.setPrintLevel(HttpLoggingInterceptor.Level.BODY);
            }
            loggingInterceptor.setColorLevel(Level.INFO);
            builder.addInterceptor(loggingInterceptor);
            builder.readTimeout(5000, TimeUnit.MILLISECONDS);      //全局的读取超时时间
            builder.writeTimeout(5000, TimeUnit.MILLISECONDS);     //全局的写入超时时间
            builder.connectTimeout(5000, TimeUnit.MILLISECONDS);   //全局的连接超时时间
            HttpsUtils.SSLParams sslParams1 = HttpsUtils.getSslSocketFactory();
            builder.sslSocketFactory(sslParams1.sSLSocketFactory, sslParams1.trustManager);
            OkGo.getInstance().init(mApp)                           //必须调用初始化
                    .setOkHttpClient(builder.build())               //建议设置OkHttpClient，不设置会使用默认的
                    .setCacheMode(CacheMode.NO_CACHE)               //全局统一缓存模式，默认不使用缓存，可以不传
                    .setCacheTime(CacheEntity.CACHE_NEVER_EXPIRE)   //全局统一缓存时间，默认永不过期，可以不传
                    .setRetryCount(3)                               //全局统一超时重连次数，默认为三次，那么最差的情况会请求4次(一次原始请求，三次重连请求)，不需要可以设置为0
                    .addCommonHeaders(headers);                     //全局公共头
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void updateToken(String token) {
        HttpHeaders headers = OkGo.getInstance().getCommonHeaders();
        headers.put("token", token);
    }

    public static String addBaseUrl(String url) {
        if (!TextUtils.isEmpty(url) && !url.startsWith("http")) {
            url = Constant.IMG_BASE_URL + url;
        }
        return url;
    }
}
