package ai.guiji.live.app.pojo;

import android.text.TextUtils;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class AudioPlayInfo implements Serializable {

    public static final int TYPE_AUDIO = 0;
    public static final int TYPE_TEXT = 1;

    public int type = TYPE_AUDIO;

    public String text;                         // 文本

    public String cacheName;                    // 名字
    public String cachePath;                    // 重采样路径缓存路径
    public long duration;                       // 时长

    public long nonce;                          // 添加一个随机数以方便在列表中匹配


    public boolean playing;     // UI上标记播放用

    public String speaker;  //tts speaker

    /**
     * 设置音频
     */
    public AudioPlayInfo(String cacheName, String cachePath, long duration) {
        type = TYPE_AUDIO;
        this.cacheName = cacheName;
        this.cachePath = cachePath;
        this.duration = duration;
    }

    /**
     * 文本设置类型
     */
    public AudioPlayInfo(String text, String cacheName, String cachePath, long duration) {
        type = TYPE_TEXT;
        this.text = text;
        this.cacheName = cacheName;
        this.cachePath = cachePath;
        this.duration = duration;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        AudioPlayInfo that = (AudioPlayInfo) o;
        return type == that.type && Objects.equals(text, that.text)
                && Objects.equals(cacheName, that.cacheName)
                && Objects.equals(cachePath, that.cachePath)
                && Objects.equals(nonce, that.nonce);
    }

    @Override
    public int hashCode() {
        return Objects.hash(type, text, cacheName, cachePath, duration, nonce);
    }

    public JSONObject buildJson() {
        JSONObject jsonObject = new JSONObject();
        try {
            if (!TextUtils.isEmpty(text)){
                jsonObject.put("text", text);
            }
            jsonObject.put("cacheName", cacheName);
            jsonObject.put("cachePath", cachePath);
            jsonObject.put("duration", duration);
            jsonObject.put("nonce", nonce);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return jsonObject;
    }

    public static List<AudioPlayInfo> fromJsonArray(JSONArray jsonArray) {
        List<AudioPlayInfo> list = new ArrayList<>();
        if (jsonArray != null) {
            try {
                for (int i = 0; i < jsonArray.length(); i++) {
                    JSONObject item = jsonArray.getJSONObject(i);
                    String text = item.optString("text");
                    String cacheName = item.optString("cacheName");
                    String cachePath = item.optString("cachePath");
                    long duration = item.optLong("duration");
                    long nonce = item.optLong("nonce");
                    File file = new File(cachePath);
                    if (file.exists()){
                        AudioPlayInfo info;
                        if (!TextUtils.isEmpty(text)){
                            info = new AudioPlayInfo(text, cacheName, cachePath, duration);
                        } else {
                            info = new AudioPlayInfo(cacheName, cachePath, duration);
                        }
                        info.nonce = nonce;
                        list.add(info);
                    }
                }
            }catch (Exception e){
                Log.e("AudioPlayInfo", "json解析异常!");
            }
        }
        return list;
    }

    @Override
    public String toString() {
        return "AudioPlayInfo{" +
                "type=" + type +
                ", text='" + text + '\'' +
                ", cacheName='" + cacheName + '\'' +
                ", cachePath='" + cachePath + '\'' +
                ", duration=" + duration +
                ", nonce=" + nonce +
                '}';
    }
}
