package ai.guiji.live.app.pojo;

import android.net.Uri;
import android.provider.MediaStore;

import java.io.Serializable;
import java.util.Objects;


/**
 * 本地媒体库查询出来的媒体信息
 */
public class MediaInfo implements Serializable {

    public enum MediaType {
        IMAGE, VIDEO, AUDIO
    }

    long id;
    String name;
    String title;
    String path;
    Uri uri;
    MediaType type;

    int width;
    int height;
    long size;

    long duration;

    long dateAdded;         // 添加时间

    private boolean select;

    public MediaInfo(MediaType type, long id, String name, String title, String path, int width, int height, long size, long dateAdded) {
        this.type = type;
        this.id = id;
        this.name = name;
        this.title = title;
        this.path = path;
        this.width = width;
        this.height = height;
        this.size = size;
        this.dateAdded = dateAdded;
        if (type == MediaType.IMAGE) {
            uri = Uri.withAppendedPath(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, String.valueOf(id));
        } else if (type == MediaType.VIDEO) {
            uri = Uri.withAppendedPath(MediaStore.Video.Media.EXTERNAL_CONTENT_URI, String.valueOf(id));
        } else if (type == MediaType.AUDIO) {
            uri = Uri.withAppendedPath(MediaStore.Audio.Media.EXTERNAL_CONTENT_URI, String.valueOf(id));
        } else {
            uri = Uri.parse(path);
        }
    }

    private MediaInfo(MediaType type, String url) {
        this.type = type;
        this.uri = Uri.parse(url);
    }

    public long getDuration() {
        return duration;
    }

    public void setDuration(long duration) {
        this.duration = duration;
    }

    public long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public MediaType getType() {
        return type;
    }

    public Uri getUri() {
        return uri;
    }

    public String getTitle() {
        return title;
    }

    public long getDateAdded() {
        return dateAdded;
    }

    public boolean isSelect() {
        return select;
    }

    public void setSelect(boolean select) {
        this.select = select;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public long getSize() {
        return size;
    }

    public void setSize(long size) {
        this.size = size;
    }

    public MediaInfo copy() {
        MediaInfo info = new MediaInfo(type, id, name, title, path, width, height, size, dateAdded);
        info.duration = duration;
        return info;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        MediaInfo mediaInfo = (MediaInfo) o;
        return id == mediaInfo.id && type == mediaInfo.type;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, type);
    }

    @Override
    public String toString() {
        return "MediaInfo{" +
                "id=" + id +
                ", type=" + type +
                ", duration=" + duration +
                ", select=" + select +
                '}';
    }
}
