package ai.guiji.live.app.ui.activity;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import ai.guiji.live.app.App;
import ai.guiji.live.app.R;
import ai.guiji.live.app.pojo.AudioFolderInfo;
import ai.guiji.live.app.pojo.MediaInfo;
import ai.guiji.live.app.ui.adapter.AudioFolderAdapter;
import ai.guiji.live.app.ui.adapter.AudioInfoAdapter;


public class AudioSelectActivity extends BaseActivity implements View.OnClickListener {

    View iv_back;
    TextView tv_title;
    View layout_select_all;
    ImageView iv_select_all;
    RecyclerView rv_folder, rv_audio;
    TextView tv_select;

    AudioFolderAdapter audioFolderAdapter;
    AudioInfoAdapter audioInfoAdapter;

    List<MediaInfo> currentFolderAudioList = new ArrayList<>();
    List<MediaInfo> mSelected = new ArrayList<>();

    @SuppressLint("NotifyDataSetChanged")
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_audio_select);
        iv_back = findViewById(R.id.iv_back);

        tv_title = findViewById(R.id.tv_title);
        layout_select_all = findViewById(R.id.layout_select_all);
        iv_select_all = findViewById(R.id.iv_select_all);
        rv_folder = findViewById(R.id.rv_folder);
        rv_audio = findViewById(R.id.rv_audio);
        tv_select = findViewById(R.id.tv_select);

        iv_back.setOnClickListener(this);
        layout_select_all.setOnClickListener(this);
        tv_select.setOnClickListener(this);

        audioInfoAdapter = new AudioInfoAdapter(mContext, item -> {
            item.setSelect(!item.isSelect());
            audioInfoAdapter.notifyDataSetChanged();
            refreshSelectAll();

            if (item.isSelect()) {
                mSelected.add(item);
            } else {
                mSelected.remove(item);
            }
            refreshSelectState();
        });
        rv_audio.setAdapter(audioInfoAdapter);

        getAllAudioFolder();
    }

    private void showFolder() {
        tv_title.setText(R.string.tv_recent_projects);
        rv_audio.setVisibility(View.GONE);
        layout_select_all.setVisibility(View.GONE);
        rv_folder.setVisibility(View.VISIBLE);
    }

    @SuppressLint("NotifyDataSetChanged")
    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.iv_back) {
            if (rv_audio.getVisibility() == View.VISIBLE) {
                showFolder();
            } else {
                finish();
            }
        } else if (id == R.id.layout_select_all) {
            if (currentFolderAudioList != null) {
                boolean selectAll = checkSelectAllState();
                if (selectAll) {
                    for (MediaInfo item : currentFolderAudioList) {
                        item.setSelect(false);
                        mSelected.remove(item);
                    }
                } else {
                    for (MediaInfo item : currentFolderAudioList) {
                        item.setSelect(true);
                        mSelected.add(item);
                    }
                }
                audioInfoAdapter.notifyDataSetChanged();
                refreshSelectAll();
                refreshSelectState();
            }
        } else if (id == R.id.tv_select) {
            List<MediaInfo> medias = new ArrayList<>(mSelected);
            App.getApp().setMedias(medias);
            Intent intent = new Intent();
            setResult(RESULT_OK, intent);
            finish();
        }
    }


    private void getAllAudioFolder() {
        Thread thread = new Thread(() -> {
            //文件夹数组
            List<AudioFolderInfo> allAudioFolderList = new ArrayList<>();
            //文件夹map
            HashMap<Long, AudioFolderInfo> bucketMap = new HashMap<>();

            // 所有音频
            AudioFolderInfo allAudioFolderInfo = new AudioFolderInfo();
            allAudioFolderInfo.setFolderId(0);
            allAudioFolderInfo.setFolderName("all audio");
            allAudioFolderInfo.setAudioList(new ArrayList<>());
            allAudioFolderList.add(0, allAudioFolderInfo);

            try (Cursor cursor = mContext.getContentResolver().query(MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
                    null, null, null, MediaStore.Audio.Media.DATE_MODIFIED + " DESC")) {
                if (cursor != null) {
                    int bucketNameColumn = cursor.getColumnIndex(MediaStore.Audio.Albums.ALBUM);
                    final int bucketIdColumn = cursor.getColumnIndex(MediaStore.Audio.Media.ALBUM_ID);
                    final int dataColumn = cursor.getColumnIndex(MediaStore.Audio.Media.DATA);
                    final int idColumn = cursor.getColumnIndex(MediaStore.Audio.Media._ID);
                    final int durationColumn = cursor.getColumnIndex(MediaStore.Audio.Media.DURATION);
                    final int sizeColumn = cursor.getColumnIndex(MediaStore.Audio.Media.SIZE);
                    final int nameColumn = cursor.getColumnIndex(MediaStore.Audio.Media.DISPLAY_NAME);
                    final int titleColumn = cursor.getColumnIndex(MediaStore.Audio.Media.TITLE);
                    final int mimeTypeColumn = cursor.getColumnIndex(MediaStore.Audio.Media.MIME_TYPE);
                    final int addColumn = cursor.getColumnIndex(MediaStore.MediaColumns.DATE_ADDED);
                    while (cursor.moveToNext()) {
                        //文件夹id
                        long bucketId = cursor.getLong(bucketIdColumn);
                        //文件夹名称
                        String bucketName = cursor.getString(bucketNameColumn);
                        //音频id
                        final long id = cursor.getLong(idColumn);
                        //音频地址
                        final String path = cursor.getString(dataColumn);
                        //音频时长
                        long duration = cursor.getLong(durationColumn);
                        //音频文件大小
                        final long size = cursor.getLong(sizeColumn);
                        //名称
                        final String name = cursor.getString(nameColumn);
                        String mimeType = cursor.getString(mimeTypeColumn);
                        final String title = cursor.getString(titleColumn);
                        final long dateAdded = cursor.getLong(addColumn);
                        if (size > 0) {
                            MediaInfo audioInfo = new MediaInfo(MediaInfo.MediaType.AUDIO, id, name, title, path, 0, 0, size, dateAdded);
                            audioInfo.setDuration(duration);
                            //添加到所有音频
                            allAudioFolderInfo.getAudioList().add(audioInfo);
                            //通过bucketId获取文件夹
                            AudioFolderInfo audioFolderInfo = bucketMap.get(bucketId);

                            if (audioFolderInfo == null) {
                                audioFolderInfo = new AudioFolderInfo();
                                audioFolderInfo.setAudioList(new ArrayList<>());
                                audioFolderInfo.setFolderId(bucketId);
                                audioFolderInfo.setFolderName(bucketName);
                                bucketMap.put(bucketId, audioFolderInfo);
                                allAudioFolderList.add(audioFolderInfo);
                            }
                            audioFolderInfo.getAudioList().add(audioInfo);
                        }
                    }
                }
            } catch (Exception e) {
                Log.e("123", "audio query error: " + e.getMessage());
            }
            setData(allAudioFolderList);
        });
        thread.setName("queryAudio");
        thread.start();
    }


    private void setData(List<AudioFolderInfo> folderList) {
        post(() -> {
            audioFolderAdapter = new AudioFolderAdapter(mContext, folderList, item -> {
                currentFolderAudioList = item.getAudioList();
                audioInfoAdapter.setData(currentFolderAudioList);
                tv_title.setText(item.getFolderName());
                rv_audio.setVisibility(View.VISIBLE);
                rv_folder.setVisibility(View.GONE);
                layout_select_all.setVisibility(View.VISIBLE);
                refreshSelectAll();
            });
            rv_folder.setAdapter(audioFolderAdapter);
        });
    }

    private void refreshSelectState() {
        if (mSelected.size() > 0) {
            tv_select.setEnabled(true);
            tv_select.setBackgroundResource(R.drawable.shape_corners_4_solid_ff444cf0);
            tv_select.setText(getString(R.string.tv_btn_add_num, mSelected.size()));
        } else {
            tv_select.setEnabled(false);
            tv_select.setBackgroundResource(R.drawable.shape_corners_4_sold_66444cf0);
            tv_select.setText(R.string.tv_btn_add);
        }
    }

    private void refreshSelectAll() {
        iv_select_all.setImageResource(checkSelectAllState() ? R.mipmap.iv_bg_list_sort_enable : R.mipmap.iv_bg_list_sort_disable);
    }

    // 看是否都选中了
    private boolean checkSelectAllState() {
        boolean all = false;
        if (currentFolderAudioList != null && currentFolderAudioList.size() > 0) {
            all = true;
            for (MediaInfo item : currentFolderAudioList) {
                if (!item.isSelect()) {
                    all = false;
                    break;
                }
            }
        }
        return all;
    }

    private void post(Runnable runnable) {
        if (null == iv_back || null == runnable) {
            return;
        }
        iv_back.post(runnable);
    }
}
