package ai.guiji.live.app.ui.activity

import ai.guiji.live.app.App
import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.pojo.AudioPlayInfo
import ai.guiji.live.app.pojo.SceneInfo
import ai.guiji.live.app.ui.dialog.LoadingLiveDialog
import ai.guiji.live.app.ui.dialog.LoadingProgressDialog
import ai.guiji.live.app.ui.dialog.RTMPAddressDialog
import ai.guiji.live.app.ui.dialog.StopLiveDialog
import ai.guiji.live.app.util.DeviceUtils
import ai.guiji.live.sdk.LiveCallback
import ai.guiji.live.sdk.LiveCore
import ai.guiji.live.sdk.LiveOptions.IMOptions
import ai.guiji.live.sdk.LiveOptions.RTCOptions
import ai.guiji.live.sdk.LiveOptions.SessionOptions
import android.media.AudioManager
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.WindowManager
import android.widget.Toast
import com.lzy.okgo.OkGo
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.activity_live.remote_renderer
import kotlinx.android.synthetic.main.activity_live.tv_stop
import org.json.JSONObject
import org.webrtc.EglBase
import org.webrtc.MediaStream
import org.webrtc.MediaStreamTrack
import org.webrtc.PeerConnection
import org.webrtc.RendererCommon
import org.webrtc.VideoTrack


class LiveActivity : BaseActivity() {

    private var eglBaseContext = EglBase.create().eglBaseContext
    private var loadingLiveDialog: LoadingLiveDialog? = null
    private var closeLiveDialog: LoadingProgressDialog? = null

    private var liveCore: LiveCore? = null

    private var roomId: Int = 0
    private var roomName: String? = null
    private var mSceneInfo: SceneInfo? = null
    private var mPlayList = ArrayList<AudioPlayInfo>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        val mAudioManager =
            mContext.getSystemService(AUDIO_SERVICE) as AudioManager
        mAudioManager.mode = AudioManager.MODE_IN_COMMUNICATION
        mAudioManager.isSpeakerphoneOn = true

        setContentView(R.layout.activity_live)
        roomId = intent.getIntExtra("roomId", 0)
        roomName = intent.getStringExtra("roomName")
        val obj = intent.getSerializableExtra("SceneInfo")
        obj?.let {
            mSceneInfo = obj as SceneInfo
        } ?: let {
            Toast.makeText(mContext, "未获取到场景信息", Toast.LENGTH_SHORT).show()
            finish()
            return
        }
//        intent.putExtra("PlayList", mPlayList)
        val list = intent.getSerializableExtra("PlayList")
        list?.let {
            mPlayList.addAll(list as List<AudioPlayInfo>)
        }

        tv_stop.setOnClickListener {
            showStopDialog()
        }
        remote_renderer.init(eglBaseContext, null)
        remote_renderer.setScalingType(RendererCommon.ScalingType.SCALE_ASPECT_FILL)
        remote_renderer.setMirror(false)
        remote_renderer.setEnableHardwareScaler(false /* enabled */)


        showLoading()
        registerTopic()
    }

    private fun registerTopic() {
        val req = JSONObject()
        req.put("liveId", roomId)
        req.put("liveName", roomName)
        OkGo.post<JSONObject>(Constant.REGISTER_TOPIC)
            .tag(TAG).upJson(req.toString())
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    val body = response?.body()
                    Log.e("123", "REGISTER_TOPIC body: $body")
                    val code = body?.optInt("code", -1)
                    val msg = body?.optString("msg")
                    val data = body?.optJSONObject("data")
                    val sessionId = data?.optString("sessionId")
                    val topicPub = data?.optString("topicPub")//客户端发送
                    val topicSub = data?.optString("topicSub")//客户端接收
                    if ((200 == code || 0 == code) && null != sessionId) {
                        connect(sessionId, topicPub, topicSub)
                    } else {
                        Toast.makeText(mContext, "msg: $msg", Toast.LENGTH_LONG).show()
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    Toast.makeText(mContext, "网络异常: ${response?.code()}", Toast.LENGTH_LONG)
                        .show()
                    dismissLoading()
                    finish()
                }
            })
    }

    private fun connect(sessionId: String, topicPub: String?, topicSub: String?) {
        mSceneInfo?.let {
            val files = ArrayList<String>()
            for (item in mPlayList) {
                files.add(item.cachePath)
            }

            val sequences = ArrayList<String>()
            sequences.add(App.addBaseUrl(it.videoCoverUrl))
            val sessionOptions =
                SessionOptions(
                    sessionId,
                    it.sceneCode,
                    sequences,
                    OkGo.getInstance().commonHeaders.get("token"),
                    "$roomId",
                    if (files.size > 0) files else null,
                    topicPub, topicSub
                ).setMatting(1 - it.sceneType)
//            // todo debug
//            sequences.add("https://digital-public-dev.obs.cn-east-3.myhuaweicloud.com/model/2023/02/23/af495409dece2355e630ea272ec94696_1080x1920_0_9.mp4")
//            val sessionOptions =
//                SessionOptions(
//                    sessionId,
//                    "389164429508677",
//                    sequences,
//                    OkGo.getInstance().commonHeaders.get("token"),
//                    "$roomId",
//                    if (files.size > 0) files else null
//                ).setMatting(1)

            val deviceId = "endpoint_android_" + DeviceUtils.getUUID(mContext)
            val imOptions = IMOptions(
                deviceId,
                Constant.MQTT_URL,
                App.getApp().mqttUsername,
                App.getApp().mqttPassword
            )
            val rtcOptions = RTCOptions()
            liveCore =
                LiveCore(
                    mContext,
                    eglBaseContext,
                    sessionOptions,
                    imOptions,
                    rtcOptions,
                    Callback()
                )
            liveCore?.connect()
        }
    }

    private fun showRTMPAddress(address: String) {
        val rtmpAddressDialog = RTMPAddressDialog(mContext, address)
        rtmpAddressDialog.show()
    }

    private fun showLoading() {
        dismissLoading()
        loadingLiveDialog =
            LoadingLiveDialog(mContext, object : LoadingLiveDialog.ClickListener {
                override fun onCancel() {
                    finish()
                }
            })
        loadingLiveDialog?.show()
        tv_stop.visibility = View.GONE
    }

    private fun dismissLoading() {
        loadingLiveDialog?.dismiss()
        tv_stop.visibility = View.VISIBLE
    }

    private fun showStopDialog() {
        val stopLiveDialog = StopLiveDialog(mContext, object : StopLiveDialog.ClickListener {
            override fun onConfirm() {
                liveCore?.sendStop()
                closeLiveDialog = LoadingProgressDialog(mContext, "关闭中")
                closeLiveDialog?.show()
                mHandler.postDelayed({
                    // 强制关闭
                    runOnUiThread {
                        Log.e("123", "强制关闭!!!")
                        closeLiveDialog?.dismiss()
                        finish()
                    }
                }, 3000)
            }
        })
        stopLiveDialog.show()
    }

    override fun onDestroy() {
        super.onDestroy()
        liveCore?.release()
        remote_renderer.release()
    }

    inner class Callback : LiveCallback {
        override fun onIMConnectFail(msg: String?) {
            runOnUiThread {
                dismissLoading()
                Toast.makeText(mContext, "onIMConnectFail: $msg", Toast.LENGTH_SHORT).show()
                finish()
            }
        }

        override fun onIMConnectLost(msg: String?) {
            Log.w("123", "onIMConnectLost $msg")
        }

        override fun onShow() {
            // 请求推送rtmp流
            liveCore?.requestPush()

            // 数字人展示出来了
            runOnUiThread {
                dismissLoading()
            }
        }

        override fun onStop(cause: String) {
            runOnUiThread {
                if (!TextUtils.isEmpty(cause) && !cause.contains("API signal destroy!")) {
                    Toast.makeText(mContext, "cause: $cause", Toast.LENGTH_SHORT).show()
                }
                mHandler.removeCallbacksAndMessages(null)
                loadingLiveDialog?.dismiss()
                closeLiveDialog?.dismiss()
                finish()
            }
        }

        override fun onPlayAddress(playaddrJson: String) {
            runOnUiThread {
                val json = JSONObject(playaddrJson)
                val rtmpJson = json.getJSONObject("rtmp")
                val wan = rtmpJson.optString("wan")
                showRTMPAddress(wan)
            }
        }

        override fun onMessage(message: String) {
            // {"id":1,"error":{"code":"500012","sessionId":"9bae3bfa-9a5b-4d50-a7d6-70ed2015e441","message":"请联系管理员检查直播服务是否异常～"}}
            Log.e("123", "接收到业务消息 onMessage $message")
        }

        override fun onIceConnectionChange(iceConnectionState: PeerConnection.IceConnectionState?) {
            Log.e("123", "onIceConnectionChange: $iceConnectionState")
        }

        override fun onAddStream(mediaStream: MediaStream?) {
            Log.e("123", "onAddStream $mediaStream")
        }

        override fun onAddTrack(track: MediaStreamTrack?, mediaStreams: Array<out MediaStream>?) {
            if (track is VideoTrack) {
                track.addSink(remote_renderer)
            }
        }

        override fun onSDPCreateError(message: String?) {
            runOnUiThread {
                dismissLoading()
                Toast.makeText(mContext, "SDPCreateError 请检查", Toast.LENGTH_SHORT).show()
                finish()
            }
        }

        override fun onSDPSetFailure(message: String?) {
            runOnUiThread {
                dismissLoading()
                Toast.makeText(mContext, "onSDPSetFailure: $message", Toast.LENGTH_SHORT).show()
                finish()
            }
        }

    }
}