package ai.guiji.live.app.ui.activity

import ai.guiji.live.app.App
import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.observer.RefreshRoomListManager
import ai.guiji.live.app.pojo.AudioPlayInfo
import ai.guiji.live.app.pojo.LiveMode
import ai.guiji.live.app.pojo.RoomInfo
import ai.guiji.live.app.pojo.SceneInfo
import ai.guiji.live.app.ui.dialog.EditLiveNameDialog
import ai.guiji.live.app.ui.dialog.LoadingProgressDialog
import ai.guiji.live.app.ui.dialog.ScriptSettingDialog
import ai.guiji.live.app.ui.dialog.SelectDigitalManDialog
import ai.guiji.live.app.util.ACache
import android.Manifest
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import com.bumptech.glide.Glide
import com.lzy.okgo.OkGo
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.activity_room_edit.*
import org.json.JSONArray
import org.json.JSONObject

class RoomEditActivity : BaseActivity() {

    private val requestReadStoragePermissionCode = 1
    private val requestAudioRecordPermissionCode = 2

    var scriptSettingDialog: ScriptSettingDialog? = null
    private var loadingProgressDialog: LoadingProgressDialog? = null

    var mPlayList = ArrayList<AudioPlayInfo>()
    var mPlayMode = LiveMode.NONE

    var mRoomInfo: RoomInfo? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_room_edit)
        val obj = intent.getSerializableExtra("RoomInfo")
        obj?.let {
            mRoomInfo = obj as RoomInfo
        } ?: let {
            Toast.makeText(mContext, "未获取到直播间信息", Toast.LENGTH_SHORT).show()
            finish()
            return
        }

        iv_back.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            finish()
        }
        layout_model.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            val selectDigitalManDialog = SelectDigitalManDialog(
                mContext,
                mRoomInfo,
                object : SelectDigitalManDialog.SelectDigitalManListener {
                    override fun onSelect(roomInfo: RoomInfo) {
                        mRoomInfo = roomInfo
                        refreshView()
                    }
                })
            selectDigitalManDialog.show()
        }

        tv_name.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            showEditNameDialog()
        }
        iv_edit_name.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            showEditNameDialog()
        }

        iv_switch_record.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            layout_mode_bar.visibility = View.INVISIBLE
            iv_enable_mode.visibility = View.VISIBLE
            iv_enable_mode.setImageResource(R.mipmap.iv_room_edit_switch_record)
            layout_script.isEnabled = true
            mPlayMode = LiveMode.RECORD
            updateCache()
        }
        iv_switch_music.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            // 这里要先设置音频话术列表
            scriptSettingDialog =
                ScriptSettingDialog(
                    mContext,
                    mPlayList,
                    object : ScriptSettingDialog.ScriptSettingListener {
                        override fun onQueryAudio() {
                            requestPermission(
                                arrayOf(
                                    Manifest.permission.READ_EXTERNAL_STORAGE,
                                ), requestReadStoragePermissionCode
                            )
                        }

                        override fun onSure(list: List<AudioPlayInfo>) {
                            mPlayList.clear()
                            mPlayList.addAll(list)
                            layout_mode_bar.visibility = View.INVISIBLE
                            iv_enable_mode.visibility = View.VISIBLE
                            iv_enable_mode.setImageResource(R.mipmap.iv_room_edit_switch_music)
                            layout_script.isEnabled = true
                            mPlayMode = LiveMode.MUSIC
                            updateCache()
                        }
                    })
            scriptSettingDialog?.show()
        }

        layout_script.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            layout_mode_bar.visibility = View.VISIBLE
            iv_enable_mode.visibility = View.INVISIBLE
            layout_script.isEnabled = false
        }

        tv_start.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
            requestPermission(
                arrayOf(Manifest.permission.RECORD_AUDIO),
                requestAudioRecordPermissionCode
            )
        }
        iv_bg.setOnClickListener {
            tv_live_mode_hint.visibility = View.GONE
        }

        // 先读缓存数据
        val cacheJson = ACache.get(mContext).getAsJSONObject("room_${mRoomInfo?.id}")
        cacheJson?.let {
            mPlayMode = LiveMode.getFromCode(cacheJson.optInt("mode", 0))
            val files = cacheJson.optJSONArray("files")
            val list = AudioPlayInfo.fromJsonArray(files)
            mPlayList.addAll(list)

            Log.e("123", "mPlayMode: $mPlayMode mPlayList: $mPlayList ")
        }

        refreshView()
    }

    override fun onPause() {
        super.onPause()
        scriptSettingDialog?.pause()
    }

    private fun showEditNameDialog() {
        val editNameDialog =
            EditLiveNameDialog(
                mContext
            ) { name ->
                name?.let {
                    editName(name)
                }
            }
        editNameDialog.show()
    }

    private fun refreshView() {
        tv_name.text = mRoomInfo?.liveName
        Glide.with(mContext).load(App.addBaseUrl(mRoomInfo?.coverUrl)).into(iv_cover)
        initModeBar()
    }

    private fun initModeBar() {
        if (mPlayMode == LiveMode.NONE) {
            layout_mode_bar.visibility = View.VISIBLE
            iv_enable_mode.visibility = View.INVISIBLE
            tv_live_mode_hint.visibility = View.VISIBLE
            layout_script.isEnabled = false
        } else {
            layout_mode_bar.visibility = View.INVISIBLE
            iv_enable_mode.visibility = View.VISIBLE
            iv_enable_mode.setImageResource(if (mPlayMode == LiveMode.MUSIC) R.mipmap.iv_room_edit_switch_music else R.mipmap.iv_room_edit_switch_record)
            tv_live_mode_hint.visibility = View.INVISIBLE
            layout_script.isEnabled = true
        }
    }

    // 更新缓存数据
    private fun updateCache() {
        val saveJson = JSONObject()
        saveJson.put("mode", mPlayMode.code)
        val list = JSONArray()
        for (item in mPlayList) {
            val json = item.buildJson()
            list.put(json)
        }
        saveJson.put("files", list)
        ACache.get(mContext).put("room_${mRoomInfo?.id}", saveJson)
    }

    private fun editName(liveName: String) {
        val url = Constant.UPDATE_LIVE
        val jsonObject = JSONObject()
        jsonObject.put("id", mRoomInfo?.id)
        jsonObject.put("liveName", liveName)
        val body = jsonObject.toString()
        OkGo.post<JSONObject>(url)
            .tag(this)
            .upJson(body)
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    response?.body()?.apply {
                        val code = optInt("code")
                        val msg = getString("msg")
                        if (code == 200) {
                            mRoomInfo?.liveName = liveName
                            refreshView()
                            RefreshRoomListManager.getInstance().notify("更新了直播间名称", null)
                            Toast.makeText(mContext, "直播间名称修改成功", Toast.LENGTH_SHORT)
                                .show()
                        } else {
                            Toast.makeText(mContext, "修改失败: $msg", Toast.LENGTH_SHORT).show()
                        }
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                }
            })
    }

    /**
     * 根据udCode查询场景信息
     */
    private fun querySceneDetails() {
        loadingProgressDialog = LoadingProgressDialog(mContext, "获取场景信息...")
        loadingProgressDialog?.show()
        OkGo.get<JSONObject>(String.format(Constant.QUERY_SCENE_DETAILS, mRoomInfo?.udCode))
            .tag(TAG)
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    loadingProgressDialog?.dismiss()
                    val body = response?.body()
                    Log.e("123", "querySceneDetailsByCode body: $body")
                    val code = body?.optInt("code")
                    val msg = body?.optString("msg")
                    val data = body?.optJSONObject("data")
                    if (code == 200 && data != null) {
                        parseScene(data)
                    } else {
                        Toast.makeText(mContext, "msg: $msg", Toast.LENGTH_SHORT).show()
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    Toast.makeText(mContext, "网络异常", Toast.LENGTH_SHORT).show()
                    loadingProgressDialog?.dismiss()
                }
            })
    }

    private fun parseScene(data: JSONObject) {
        val sceneInfo = SceneInfo()
        sceneInfo.id = data.optInt("id")
        sceneInfo.sceneName = data.optString("sceneName")
        sceneInfo.sceneCode = data.optString("sceneCode")
        sceneInfo.sceneType = data.optInt("sceneType")
        sceneInfo.coverUrl = data.optString("coverUrl")
        sceneInfo.videoCoverUrl = data.optString("videoCoverUrl")
        sceneInfo.originalModelPath = data.optString("originalModelPath")
        val intent = Intent(mContext, LiveActivity::class.java)
        intent.putExtra("roomId", mRoomInfo?.id)
        intent.putExtra("roomName", mRoomInfo?.liveName)
        intent.putExtra("SceneInfo", sceneInfo)
        if (mPlayMode == LiveMode.MUSIC && mPlayList.size > 0) {
            intent.putExtra("PlayList", mPlayList)
        }
        startActivity(intent)
        finish()
    }

    private var audioMediaLauncher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == RESULT_OK) {
                result.data?.let {
                    scriptSettingDialog?.transfer()
                }
            }
        }

    override fun permissionsGet(get: Boolean, code: Int) {
        super.permissionsGet(get, code)
        if (get) {
            if (code == requestReadStoragePermissionCode) {
                audioMediaLauncher.launch(Intent(mContext, AudioSelectActivity::class.java))
            } else if (code == requestAudioRecordPermissionCode) {
                if (mPlayMode == LiveMode.NONE) {
                    Toast.makeText(mContext, "请选择直播话术", Toast.LENGTH_SHORT).show()
                } else {
                    querySceneDetails()
                }
            }
        }
    }
}