package ai.guiji.live.app.ui.activity

import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.observer.IObserver
import ai.guiji.live.app.observer.RefreshRoomListManager
import ai.guiji.live.app.pojo.RoomInfo
import ai.guiji.live.app.ui.adapter.RoomAdapter
import ai.guiji.live.app.ui.dialog.SelectDigitalManDialog
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.Toast
import com.lzy.okgo.OkGo
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.activity_room_list.*
import kotlinx.android.synthetic.main.activity_room_list.layout_empty
import org.json.JSONObject

class RoomListActivity : BaseActivity(), IObserver<Any> {

    private val mList = ArrayList<RoomInfo>()
    var roomAdapter: RoomAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_room_list)
        setStatueBarMode(true)

        layout_create.setOnClickListener {
            val selectDigitalManDialog = SelectDigitalManDialog(
                mContext,
                null,
                object : SelectDigitalManDialog.SelectDigitalManListener {
                    override fun onSelect(roomInfo: RoomInfo) {
                        // 直接跳转到编辑页面吗?
                        val intent = Intent(mContext, RoomEditActivity::class.java)
                        intent.putExtra("RoomInfo", roomInfo)
                        startActivity(intent)
                    }
                })
            selectDigitalManDialog.show()
        }
        iv_refresh.setOnClickListener {
            refreshData()
        }

        layout_refresh.setOnRefreshListener {
            refreshData()
        }

        roomAdapter = RoomAdapter(mContext, object : RoomAdapter.RoomListener {
            override fun onClickItem(position: Int, item: RoomInfo) {
                val intent = Intent(mContext, RoomEditActivity::class.java)
                intent.putExtra("RoomInfo", item)
                startActivity(intent)
            }
        })
        rv_live.adapter = roomAdapter
        refreshData()

        RefreshRoomListManager.getInstance().add(this)
    }

    private fun refreshData() {
        mList.clear()
        layout_refresh.isRefreshing = true
        val json = JSONObject()
        json.put("page", 1)
        json.put("size", 100)
        json.put("liveType", 0)
        OkGo.post<JSONObject>(
            String.format(
                Constant.API_LIST_LIVE,
                OkGo.getInstance().commonHeaders.get("token")
            )
        )
            .tag(TAG)
            .upJson(json.toString())
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    layout_refresh.isRefreshing = false
                    val body = response?.body()
                    Log.e("123", "listLive body: $body")
                    val code = body?.optInt("code")
                    val msg = body?.optString("msg")
                    val data = body?.optJSONObject("data")
                    if (code == 200 && data != null) {
                        parseData(data)
                    } else {
                        Toast.makeText(mContext, "listLive msg: $msg", Toast.LENGTH_SHORT).show()
                        roomAdapter?.setData(null)
                        layout_empty.visibility = View.VISIBLE
                        // 100502 是token异常

                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    Toast.makeText(mContext, "网络异常", Toast.LENGTH_SHORT).show()
                    layout_refresh.isRefreshing = false
                    roomAdapter?.setData(null)
                    layout_empty.visibility = View.VISIBLE
                }
            })
    }

    private fun parseData(data: JSONObject) {
        val records = data.optJSONArray("records")
        records?.let {
            for (index in 0 until records.length()) {
                val info = RoomInfo()
                val item = records.getJSONObject(index)
                info.id = item.optInt("id")
                info.liveName = item.optString("liveName")
                info.liveType = item.optInt("liveType")
                info.udCode = item.optString("udCode")
                info.robotCode = item.optString("robotCode")
                info.robotName = item.optString("robotName")
                info.coverUrl = item.optString("coverUrl")
                info.updatedTime = item.optString("updatedTime")
                info.del = item.optInt("del")
                if (info.del == 0 && !TextUtils.isEmpty(info.udCode) && "null" != info.udCode) {
                    mList.add(info)
                }
            }
        }
        roomAdapter?.setData(mList)
        layout_empty.visibility = if (mList.size == 0) View.VISIBLE else View.INVISIBLE
    }

    override fun onDestroy() {
        super.onDestroy()
        RefreshRoomListManager.getInstance().remove(this)
    }

    override fun update(message: String?, obj: Any?) {
        runOnUiThread {
            refreshData()
        }
    }

}