package ai.guiji.live.app.ui.activity

import ai.guiji.live.app.App
import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.ui.dialog.LoadingProgressDialog
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.widget.Toast
import com.lzy.okgo.OkGo
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.activity_validate.*
import org.json.JSONObject

class ValidateActivity : BaseActivity() {

    private var loadingProgressDialog: LoadingProgressDialog? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_validate)

        btn_login.setOnClickListener {
            val appId = et_app_id.text.toString()
            val appSecret = et_app_secret.text.toString()
            if (!TextUtils.isEmpty(appId) && !TextUtils.isEmpty(appSecret)) {
                requestToken(appId, appSecret)
            } else {
                Toast.makeText(mContext, "请输入appId和appSecret信息", Toast.LENGTH_SHORT).show()
            }
        }
    }

    private fun requestToken(appId: String, appSecret: String) {
        loadingProgressDialog = LoadingProgressDialog(mContext, "获取场景信息...")
        loadingProgressDialog?.show()

        val json = JSONObject()
        json.put("appId", appId)
        json.put("appSecret", appSecret)
        Log.e("123", "requestToken body: $json")
        OkGo.post<JSONObject>(Constant.API_ACCESS_TOKEN)
            .tag(TAG)
            .upJson(json.toString())
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    loadingProgressDialog?.dismiss()
                    val body = response?.body()
                    Log.e("123", "requestToken body: $body")
                    val code = body?.optInt("code")
                    val data = body?.optJSONObject("data")
                    val msg = body?.optString("msg")
                    data?.optString("mqttUser")?.let {
                        App.getApp().mqttUsername = it
                    }
                    data?.optString("mqttPassword")?.let {
                        App.getApp().mqttPassword = it
                    }
                    if (code == 200 && data != null) {
                        val token = data.optString("token")
                        App.getApp().updateToken(token)
                        Log.d("123", "获取Token成功: $data")
                        Toast.makeText(mContext, "获取Token成功: $data", Toast.LENGTH_SHORT).show()
                        startActivity(Intent(mContext, RoomListActivity::class.java))
                        finish()
                    } else {
                        Toast.makeText(mContext, "获取Token失败 msg: $msg", Toast.LENGTH_SHORT)
                            .show()
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    loadingProgressDialog?.dismiss()
                    Toast.makeText(
                        mContext,
                        "获取Token异常 code: ${response?.code()} msg: ${response?.message()}",
                        Toast.LENGTH_SHORT
                    ).show()
                }
            })
    }

}