package ai.guiji.live.app.ui.adapter;

import android.annotation.SuppressLint;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.List;

import ai.guiji.live.app.R;
import ai.guiji.live.app.pojo.MediaInfo;
import ai.guiji.live.app.util.ClickListenerUtil;
import ai.guiji.live.app.util.StringUtils;


/**
 * 文件名：AudioInfoAdapter
 * 描述：音频信息
 */
public class AudioInfoAdapter extends RecyclerView.Adapter<AudioInfoAdapter.PagerAdapterHolder> {

    Context mContext;
    List<MediaInfo> mAudioList;
    AudioClickListener listener;

    public AudioInfoAdapter(Context context, AudioClickListener listener) {
        mContext = context;
        this.listener = listener;
    }

    @SuppressLint("NotifyDataSetChanged")
    public void setData(List<MediaInfo> audioList) {
        this.mAudioList = audioList;
        notifyDataSetChanged();
    }

    @NonNull
    @Override
    public PagerAdapterHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new PagerAdapterHolder(LayoutInflater.from(mContext)
                .inflate(R.layout.item_audio_item, parent, false)
        );
    }

    @Override
    public void onBindViewHolder(@NonNull PagerAdapterHolder holder, int position) {
        MediaInfo bean = mAudioList.get(position);

        //名称
        holder.tv_name.setText(bean.getName());
        //时间
        holder.tv_time.setText(StringUtils.formatVideoDateTime((long) Math.floor(bean.getDuration() / 1000.0)));
        //大小
        holder.tv_size.setText(StringUtils.formatSize(bean.getSize()));
        holder.iv_select.setSelected(bean.isSelect());

        //根布局点击
        holder.itemView.setOnClickListener(v -> {
            if (!ClickListenerUtil.canClick(200)) return;
            if (null != listener) {
                listener.onClick(bean);
            }
        });

    }

    @Override
    public int getItemCount() {
        return mAudioList == null ? 0 : mAudioList.size();
    }

    static class PagerAdapterHolder extends RecyclerView.ViewHolder {

        ImageView img_audio;
        TextView tv_name, tv_time, tv_size;
        View iv_select;

        public PagerAdapterHolder(@NonNull View itemView) {
            super(itemView);
            img_audio = itemView.findViewById(R.id.img_audio);
            tv_name = itemView.findViewById(R.id.tv_name);
            tv_time = itemView.findViewById(R.id.tv_time);
            tv_size = itemView.findViewById(R.id.tv_size);
            iv_select = itemView.findViewById(R.id.iv_select);
        }
    }

    public interface AudioClickListener {
        void onClick(MediaInfo item);
    }
}
