package ai.guiji.live.app.ui.dialog

import ai.guiji.live.app.App
import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.pojo.AudioPlayInfo
import ai.guiji.live.app.pojo.TimbreInfo
import ai.guiji.live.app.ui.activity.BaseActivity
import ai.guiji.live.app.ui.adapter.TimbreAdapter
import ai.guiji.live.sdk.audio.AudioConvert
import ai.guiji.live.sdk.util.CommonMD5Util
import android.app.Dialog
import android.os.Bundle
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.util.Log
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.Toast
import com.bumptech.glide.Glide
import com.lzy.okgo.OkGo
import com.lzy.okgo.callback.FileCallback
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.dialog_add_text_script.et_content
import kotlinx.android.synthetic.main.dialog_add_text_script.iv_close
import kotlinx.android.synthetic.main.dialog_add_text_script.iv_timbre
import kotlinx.android.synthetic.main.dialog_add_text_script.layout_list_cancel
import kotlinx.android.synthetic.main.dialog_add_text_script.layout_select_timbre
import kotlinx.android.synthetic.main.dialog_add_text_script.layout_timbre
import kotlinx.android.synthetic.main.dialog_add_text_script.rv_timbre
import kotlinx.android.synthetic.main.dialog_add_text_script.tv_cancel
import kotlinx.android.synthetic.main.dialog_add_text_script.tv_confirm
import kotlinx.android.synthetic.main.dialog_add_text_script.tv_text_content_clear
import kotlinx.android.synthetic.main.dialog_add_text_script.tv_text_content_length
import kotlinx.android.synthetic.main.dialog_add_text_script.tv_timbre
import org.json.JSONArray
import org.json.JSONObject
import java.io.File


class AddTextScriptDialog(
    private val mContext: BaseActivity,
    val mInfo: AudioPlayInfo?,
    var mListener: AddTexScriptListener
) : Dialog(mContext, R.style.dialog_bottom) {

    private val mList = ArrayList<TimbreInfo>()
    private var mSelect: TimbreInfo? = null
    private var timbreAdapter: TimbreAdapter? = null

    private var loadingProgressDialog: LoadingProgressDialog? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        setContentView(R.layout.dialog_add_text_script)

        iv_close.setOnClickListener {
            dismiss()
        }
        layout_select_timbre.setOnClickListener {
            layout_list_cancel.visibility = View.VISIBLE
            layout_timbre.visibility = View.VISIBLE
        }
        tv_cancel.setOnClickListener {
            dismiss()
        }
        tv_confirm.setOnClickListener {
            ttsSynthesis()
        }
        tv_text_content_clear.setOnClickListener {
            et_content.setText("")
        }
        layout_list_cancel.setOnClickListener {
            layout_list_cancel.visibility = View.GONE
            layout_timbre.visibility = View.GONE
        }

        et_content.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
            }

            override fun afterTextChanged(s: Editable?) {
                refreshContentLength()
            }
        })

        timbreAdapter = TimbreAdapter(mContext, object : TimbreAdapter.TimbreListener {
            override fun onSelect(position: Int, item: TimbreInfo) {
                mSelect = item
                layout_list_cancel.visibility = View.GONE
                layout_timbre.visibility = View.GONE
                refreshSelect()
            }
        })
        rv_timbre.adapter = timbreAdapter

        window?.let {
            it.setGravity(Gravity.BOTTOM) //设置显示在底部
            it.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        }
        setCancelable(true)
        setCanceledOnTouchOutside(true)
        mInfo?.let {
            et_content.setText(it.text)
        }
        refreshContentLength()
        queryTimbre()
    }

    private fun ttsSynthesis() {
        val content = et_content.text.toString()
        if (TextUtils.isEmpty(content)) {
            Toast.makeText(mContext, "请输入文本内容", Toast.LENGTH_SHORT).show()
            return
        }

        mSelect?.let {
            loadingProgressDialog = LoadingProgressDialog(mContext, "音频合成中...")
            loadingProgressDialog?.show()
            val json = JSONObject()
            json.put("md5Sum", "")
            json.put("upload", false)
            val customTTSReq = JSONObject()
            customTTSReq.put("text", content)
            customTTSReq.put("pitch", it.intonation)
            customTTSReq.put("eightK", false)
            customTTSReq.put("volume", it.volume)
            customTTSReq.put("speedRate", it.speed)
            customTTSReq.put("samplerate", "16000")
            customTTSReq.put("ttsSpeaker", it.ttsSpeaker)
            customTTSReq.put("neuralEngine", false)
            json.put("customTTSReq", customTTSReq)
            json.put("engineTypeCode", 10)
            OkGo.post<JSONObject>(Constant.API_SYNC_SYNTHESIS_WAV_CHANGE_VOICE)
                .tag("AddTextScriptDialog")
                .upJson(json.toString())
                .execute(object : JsonCallback<JSONObject>() {
                    override fun onSuccess(response: Response<JSONObject>?) {
                        val body = response?.body()
                        Log.e("123", "tts sync synthesis body: $body")
                        val code = body?.optInt("code")
                        val msg = body?.optString("msg")
                        val data = body?.optJSONObject("data")
                        if (code == 200 && data != null) {
                            val filePath = data.optString("filePath")
                            val duration = data.optLong("duration")

                            val cacheAudioDir =
                                File(File(context.externalCacheDir, "guiji"), "audio")
                            val name = "tts_" + CommonMD5Util.md5Encode(filePath) + ".tmp"
                            val cacheAudio = File(
                                cacheAudioDir,
                                name
                            )
                            OkGo.get<File>(filePath).tag("AddTextScriptDialog")
                                .execute(object : FileCallback(cacheAudioDir.absolutePath, name) {
                                    override fun onSuccess(response: Response<File>?) {
                                        Thread {
                                            val path = AudioConvert.audioResample4Path(
                                                mContext,
                                                cacheAudio.absolutePath
                                            )
                                            path?.let {
                                                Log.e("123", "转码完成 path: $path")
                                                val info =
                                                    AudioPlayInfo(
                                                        content,
                                                        "",
                                                        path,
                                                        duration
                                                    )
                                                info.nonce =
                                                    (Math.random() * Long.MAX_VALUE).toLong()
                                                info.speaker = mSelect?.ttsSpeaker
                                                mContext.runOnUiThread {
                                                    loadingProgressDialog?.dismiss()
                                                    mListener.onSure(info)
                                                    dismiss()
                                                }
                                            } ?: let {
                                                mContext.runOnUiThread {
                                                    loadingProgressDialog?.dismiss()
                                                    Toast.makeText(
                                                        mContext,
                                                        "音频转码失败",
                                                        Toast.LENGTH_SHORT
                                                    )
                                                        .show()
                                                }
                                            }
                                        }.start()
                                    }

                                    override fun onError(response: Response<File>?) {
                                        super.onError(response)
                                        loadingProgressDialog?.dismiss()
                                        Toast.makeText(mContext, "音频下载失败", Toast.LENGTH_SHORT)
                                            .show()
                                    }
                                })
                        } else {
                            loadingProgressDialog?.dismiss()
                            Toast.makeText(mContext, "TTS合成失败 msg: $msg", Toast.LENGTH_SHORT)
                                .show()
                        }
                    }

                    override fun onError(response: Response<JSONObject>?) {
                        super.onError(response)
                        Toast.makeText(mContext, "TTS合成失败", Toast.LENGTH_SHORT).show()
                        loadingProgressDialog?.dismiss()
                    }
                })
        } ?: let {
            Toast.makeText(mContext, "请选择音色", Toast.LENGTH_SHORT).show()
        }
    }

    private fun queryTimbre() {
        mList.clear()

        val jsonObject = JSONObject()
        jsonObject.put("pageNum", 1)
        jsonObject.put("pageSize", 100)
        jsonObject.put("userCategory", "")
        val params = JSONObject()
        params.put("productType", 10002)
        params.put("corpId", "")
        jsonObject.put("params", params)
        val include = JSONArray()
        include.put(10)
        jsonObject.put("includeTtsSource", include)
        jsonObject.put("isAllocation", 0)
        jsonObject.put("queryLanguage", "cn")
        val url = String.format(
            Constant.API_LIST_TTS_MODEL,
            OkGo.getInstance().commonHeaders.get("token")
        )
        val body = jsonObject.toString()
//        Log.e("123", "queryTimbre: url: $url body: $body")
        OkGo.post<JSONObject>(url)
            .tag(this)
            .upJson(body)
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    mList.clear()
                    response?.body()?.apply {
//                        Log.e("123", "query tts dub result $this")
                        val code = optInt("code")
                        val msg = optString("msg")
                        if (code == 200) {
                            val data = optJSONObject("data")
                            data?.apply {
                                val list = getJSONArray("list")
                                for (i in 0 until list.length()) {
                                    val item = list.optJSONObject(i)
                                    // parseItem(item)
                                    val timbre = TimbreInfo.parseJson(item)
                                    mList.add(timbre)
                                }
                            }
                        } else {
                            Toast.makeText(mContext, msg, Toast.LENGTH_SHORT).show()
                        }
                        timbreAdapter?.setData(mList)
                        if (mList.size > 0) {
                            mInfo?.speaker?.let { speaker ->
                                mSelect = mList.find { it.ttsSpeaker == speaker }
                                mSelect ?: let { mSelect = mList[0] }
                            } ?: let {
                                mSelect = mList[0]
                            }

                        }
                        refreshSelect()
                    }

                }

            })
    }

    private fun refreshContentLength() {
        val content = et_content.text.toString()
        tv_text_content_length.text =
            mContext.getString(R.string.total_words, content.length)
    }

    private fun refreshSelect() {
        mSelect?.let {
            Glide.with(mContext).load(App.addBaseUrl(it.ttsCover)).into(iv_timbre)
            tv_timbre.text = it.name
        }
    }

    interface AddTexScriptListener {
        fun onSure(info: AudioPlayInfo)
    }

}