package ai.guiji.live.app.ui.dialog;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.text.InputFilter;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.Toast;

import androidx.annotation.NonNull;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ai.guiji.live.app.R;
import ai.guiji.live.app.util.ClickListenerUtil;


/**
 * 背景组名称编辑
 */
public class EditLiveNameDialog extends Dialog implements View.OnClickListener {

    Context mContext;
    private ClickListener mClickListener;
    EditText et_name;
    View tv_cancel, tv_sure;

    public EditLiveNameDialog(@NonNull Context context, ClickListener clickListener) {
        super(context, R.style.dialog_bottom);
        mContext = context;
        mClickListener = clickListener;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        View contentView = getLayoutInflater().inflate(R.layout.dialog_edit_live_name, null);
        setContentView(contentView);

        et_name = findViewById(R.id.et_name);
        tv_cancel = findViewById(R.id.tv_cancel);
        tv_sure = findViewById(R.id.tv_sure);
        tv_cancel.setOnClickListener(this);
        tv_sure.setOnClickListener(this);

        //限制只能输入中文，英文，数字
        InputFilter typeFilter = (source, start, end, dest, dstart, dend) -> {
            Pattern p = Pattern.compile("[0-9a-zA-Z|\u4e00-\u9fa5]+");
            Matcher m = p.matcher(source.toString());
            if (!m.matches()) return "";
            return null;
        };
        et_name.setFilters(new InputFilter[]{typeFilter, new InputFilter.LengthFilter(10)});

        getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        setCancelable(false);
        setCanceledOnTouchOutside(false);
    }

    @Override
    public void onClick(View v) {
        if (!ClickListenerUtil.canClick()) return;
        int id = v.getId();
        if (id == R.id.tv_cancel) {
            dismiss();
        } else if (id == R.id.tv_sure) {
            String name = et_name.getText().toString();
            if (!TextUtils.isEmpty(name)) {
                dismiss();
                mClickListener.onRename(name);
            } else {
                Toast.makeText(mContext, "直播间名字不能为空", Toast.LENGTH_SHORT).show();
            }
        }
    }

    public interface ClickListener {
        void onRename(String name);
    }
}
