package ai.guiji.live.app.ui.dialog

import ai.guiji.live.app.App
import ai.guiji.live.app.R
import ai.guiji.live.app.pojo.AudioPlayInfo
import ai.guiji.live.app.ui.activity.BaseActivity
import ai.guiji.live.app.ui.adapter.ScriptAdapter
import ai.guiji.live.app.ui.view.itemtouch.ItemTouchHelperCallback
import ai.guiji.live.sdk.audio.AudioConvert
import android.app.Dialog
import android.media.MediaPlayer
import android.os.Bundle
import android.util.Log
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.Toast
import androidx.recyclerview.widget.ItemTouchHelper
import androidx.recyclerview.widget.RecyclerView
import kotlinx.android.synthetic.main.dialog_script_setting.*
import java.util.*


class ScriptSettingDialog(
    private val mContext: BaseActivity,
    private val mOriList: ArrayList<AudioPlayInfo>,
    var mListener: ScriptSettingListener
) : Dialog(mContext, R.style.dialog_bottom) {

    private val mPlayList = ArrayList<AudioPlayInfo>()
    private var scriptAdapter: ScriptAdapter? = null

    private var addTextScriptDialog: AddTextScriptDialog? = null
    private var loadingProgressDialog: LoadingProgressDialog? = null
    private var mMediaPlayer: MediaPlayer? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        setContentView(R.layout.dialog_script_setting)

        iv_close.setOnClickListener {
            dismiss()
        }
        tv_add.setOnClickListener {
            val scriptTypeDialog =
                ScriptTypeDialog(mContext, object : ScriptTypeDialog.ScriptTypeListener {
                    override fun onAudio() {
                        mListener.onQueryAudio()
                    }

                    override fun onText() {
                        editText()
                    }
                })
            scriptTypeDialog.show()
        }
        tv_confirm.setOnClickListener {
            if (mPlayList.size > 0) {
                mListener.onSure(mPlayList)
                dismiss()
            } else {
                Toast.makeText(mContext, "请添加话术", Toast.LENGTH_SHORT).show()
            }
        }

        scriptAdapter = ScriptAdapter(mContext, object : ScriptAdapter.ScriptListener {
            override fun onRemove(position: Int, info: AudioPlayInfo) {
                if (info.playing) {
                    if (mMediaPlayer?.isPlaying == true) {
                        mMediaPlayer?.stop()
                    }
                    stopRefresh()
                }
                val iterator = mPlayList.iterator()
                while (iterator.hasNext()) {
                    val item = iterator.next()
                    if (info == item) {
//                        mPlayList.remove(item)
                        iterator.remove()
                    }
                }
                scriptAdapter?.setData(mPlayList)
            }

            override fun onSelect(position: Int, info: AudioPlayInfo) {
                if (!info.playing) {
                    for (item in mPlayList) {
                        item.playing = false
                    }
                    info.playing = true
                    if (mMediaPlayer?.isPlaying == true) {
                        mMediaPlayer?.stop()
                    }
                    mMediaPlayer?.reset()
                    mMediaPlayer?.setDataSource(info.cachePath)
                    mMediaPlayer?.prepareAsync()
                } else {
                    for (item in mPlayList) {
                        item.playing = false
                    }
                    if (mMediaPlayer?.isPlaying == true) {
                        mMediaPlayer?.stop()
                    }
                }
                scriptAdapter?.refresh()
            }

            override fun onEdit(position: Int, info: AudioPlayInfo) {
                editText(info)
            }
        })
        rv_script.adapter = scriptAdapter
        val itemTouchHelper = ItemTouchHelper(object : ItemTouchHelperCallback() {
            override fun onMove(
                recyclerView: RecyclerView,
                viewHolder: RecyclerView.ViewHolder,
                target: RecyclerView.ViewHolder
            ): Boolean {
                //得到当拖拽的viewHolder的Position
                val fromPosition: Int = viewHolder.bindingAdapterPosition
                //拿到当前拖拽到的item的viewHolder
                val toPosition: Int = target.bindingAdapterPosition
                if (fromPosition < toPosition) {
                    for (i in fromPosition until toPosition) {
                        Collections.swap(mPlayList, i, i + 1)
                    }
                } else {
                    for (i in fromPosition downTo toPosition + 1) {
                        Collections.swap(mPlayList, i, i - 1)
                    }
                }
                scriptAdapter?.notifyItemMoved(fromPosition, toPosition)
                return super.onMove(recyclerView, viewHolder, target)
            }
        })
        itemTouchHelper.attachToRecyclerView(rv_script)

        mMediaPlayer = MediaPlayer()
        mMediaPlayer?.isLooping = false
        mMediaPlayer?.setOnPreparedListener {
            if (isShowing) {
                mMediaPlayer?.start()
            }
        }
        mMediaPlayer?.setOnCompletionListener {
            Log.e("123", "播放完成")
            mContext.runOnUiThread {
                stopRefresh()
            }
        }
        mMediaPlayer?.setOnErrorListener { _, what, extra ->
            Log.e("123", "播放异常: what: $what extra: $extra")
            mContext.runOnUiThread {
                stopRefresh()
            }
            false
        }
        mOriList.let {
            mPlayList.clear()
            mPlayList.addAll(mOriList)
            scriptAdapter?.setData(mPlayList)
        }

        layout_empty.visibility = if (mPlayList.size > 0) View.INVISIBLE else View.VISIBLE

        window?.let {
            it.setGravity(Gravity.BOTTOM) //设置显示在底部
            it.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        }
        setCancelable(true)
        setCanceledOnTouchOutside(true)
    }

    private fun editText(inputInfo: AudioPlayInfo? = null) {
        if (addTextScriptDialog?.isShowing == true) {
            addTextScriptDialog?.dismiss()
        }
        addTextScriptDialog = AddTextScriptDialog(
            mContext, inputInfo,
            object : AddTextScriptDialog.AddTexScriptListener {
                override fun onSure(info: AudioPlayInfo) {
                    inputInfo?.let {
                        val index = mPlayList.indexOf(inputInfo)
                        mPlayList.removeAt(index)
                        mPlayList.add(index, info)
                    } ?: let {
                        mPlayList.add(info)
                    }
                    scriptAdapter?.setData(mPlayList)
                    layout_empty.visibility =
                        if (mPlayList.size > 0) View.INVISIBLE else View.VISIBLE
                }
            })
        addTextScriptDialog?.show()
    }

    fun stopRefresh() {
        for (item in mPlayList) {
            item.playing = false
        }
        scriptAdapter?.refresh()
    }

    fun transfer() {
        loadingProgressDialog = LoadingProgressDialog(mContext)
        loadingProgressDialog?.show()
        Thread {
            val list = App.getApp().medias
            var transferHasError = false
            val selectList = ArrayList<AudioPlayInfo>()
            for (item in list) {
                Log.e("123", "开始转码")
                val path = AudioConvert.audioResample4Uri(mContext, item.uri)
                path?.let {
                    Log.e("123", "转码完成 path: $path")
                    val info = AudioPlayInfo(
                        item.title,
                        path,
                        item.duration
                    )
                    info.nonce = (Math.random() * Long.MAX_VALUE).toLong()
                    selectList.add(info)
                } ?: let {
                    transferHasError = true
                }
            }
            mContext.runOnUiThread {
                loadingProgressDialog?.dismiss()
                if (transferHasError) {
                    Toast.makeText(mContext, "转码异常", Toast.LENGTH_SHORT).show()
                } else {
                    mPlayList.addAll(selectList)
                    scriptAdapter?.setData(mPlayList)
                    layout_empty.visibility =
                        if (mPlayList.size > 0) View.INVISIBLE else View.VISIBLE
                }
            }

        }.start()
    }

    fun pause() {
        stopRefresh()
        try {
            if (mMediaPlayer?.isPlaying == true) {
                mMediaPlayer?.stop()
            }
        } catch (e: Exception) {

        }
    }

    override fun dismiss() {
        super.dismiss()
        try {
            if (mMediaPlayer?.isPlaying == true) {
                mMediaPlayer?.stop()
            }
            mMediaPlayer?.release()
        } catch (e: Exception) {
        }
    }

    interface ScriptSettingListener {
        fun onQueryAudio()
        fun onSure(list: List<AudioPlayInfo>)
    }

}