package ai.guiji.live.app.ui.dialog

import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.observer.RefreshRoomListManager
import ai.guiji.live.app.pojo.DigitalManInfo
import ai.guiji.live.app.pojo.RoomInfo
import ai.guiji.live.app.ui.activity.BaseActivity
import ai.guiji.live.app.ui.adapter.SelectDigitalManTabAdapter
import ai.guiji.live.app.ui.fragment.SelectDigitalManFragment
import ai.guiji.live.app.util.StringUtils
import android.app.Dialog
import android.os.Bundle
import android.util.Log
import android.view.Gravity
import android.view.ViewGroup
import android.view.Window
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.viewpager2.adapter.FragmentStateAdapter
import androidx.viewpager2.widget.ViewPager2
import com.google.gson.Gson
import com.google.gson.JsonParser
import com.lzy.okgo.OkGo
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.dialog_select_digital_man.*
import org.json.JSONObject


class SelectDigitalManDialog(
    private val mContext: BaseActivity,
    var roomInfo: RoomInfo?,
    var mListener: SelectDigitalManListener
) : Dialog(mContext, R.style.dialog_bottom) {

    val TAG = javaClass.name

    private var mSelect: DigitalManInfo? = null
    private var tabAdapter: SelectDigitalManTabAdapter? = null

    private var loadingProgressDialog: LoadingProgressDialog? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        setContentView(R.layout.dialog_select_digital_man)

        iv_close.setOnClickListener {
            dismiss()
        }

        tv_confirm.setOnClickListener {
            mSelect?.let { select ->
                createOrUpdate(select)
            } ?: let {
                Toast.makeText(mContext, "请选择一个数字人", Toast.LENGTH_SHORT).show()
            }
        }

        val tags = ArrayList<String>()
        tags.add("推荐")
        tags.add("我的")

        tabAdapter = SelectDigitalManTabAdapter(mContext,
            tags,
            object : SelectDigitalManTabAdapter.Listener {
                override fun onItemClick(position: Int) {
                    vp_digital.currentItem = position
                }
            })
        rv_tab.adapter = tabAdapter

        vp_digital.adapter =
            object : FragmentStateAdapter(mContext.supportFragmentManager, mContext.lifecycle) {
                override fun createFragment(position: Int): Fragment {
                    val fragment = SelectDigitalManFragment()
                    fragment.setListener(object :
                        SelectDigitalManFragment.SelectDigitalManFragmentListener {
                        override fun onSelect(item: DigitalManInfo) {
                            if (!item.select) {
                                mSelect = item
                                for (itemFragment in mContext.supportFragmentManager.fragments) {
                                    if (itemFragment is SelectDigitalManFragment) {
                                        itemFragment.refreshSelect(item)
                                    }
                                }
                            }
                        }
                    })
                    val bundle = Bundle()
                    bundle.putInt("position", position)
                    fragment.arguments = bundle
                    return fragment
                }

                override fun getItemCount(): Int {
                    return tags.size
                }
            }
        vp_digital.registerOnPageChangeCallback(object : ViewPager2.OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                super.onPageSelected(position)
                tabAdapter?.setPosition(position)
            }
        })

        window?.let {
            it.setGravity(Gravity.BOTTOM) //设置显示在底部
            it.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        }
        setCancelable(true)
        setCanceledOnTouchOutside(true)
    }

    private fun createOrUpdate(select: DigitalManInfo) {
        loadingProgressDialog = LoadingProgressDialog(
            mContext, if (null == roomInfo) "直播间创建中..."
            else "直播间修改中..."
        )
        loadingProgressDialog?.show()

        val req = RoomInfo()
        roomInfo?.let {
            req.id = it.id
            req.liveType = it.liveType
            req.liveName = it.liveName
        } ?: let {
            val liveName = "数字人直播_${StringUtils.dateToStringMS3()}"
            req.liveType = 0
            req.liveName = liveName
        }
        req.coverUrl = select.sceneList[0].coverUrl
        req.udCode = select.sceneList[0].sceneCode
        req.robotCode = select.robotCode
        req.robotName = select.robotName
        OkGo.post<JSONObject>(Constant.CREATE_OR_UPDATE_LIVE).tag(TAG).upJson(Gson().toJson(req))
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    val body = response?.body()
                    Log.e("123", "create body: $body")
                    val code = body?.optInt("code")
                    val msg = body?.optString("msg")
                    val id = body?.optInt("data")
                    if (code == 200) {
                        RefreshRoomListManager.getInstance().notify("新建了直播间", null)
                        loadingProgressDialog?.dismiss()
                        dismiss()
                        roomInfo ?: let { req.id = id ?: 0 }
                        mListener.onSelect(req)
                    } else {
                        Toast.makeText(mContext, "msg: $msg", Toast.LENGTH_SHORT).show()
                        loadingProgressDialog?.dismiss()
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    Toast.makeText(mContext, "网络异常", Toast.LENGTH_SHORT).show()
                    loadingProgressDialog?.dismiss()
                }
            })
    }

    interface SelectDigitalManListener {
        fun onSelect(roomInfo: RoomInfo)
    }

}