package ai.guiji.live.app.ui.fragment

import ai.guiji.live.app.App
import ai.guiji.live.app.Constant
import ai.guiji.live.app.R
import ai.guiji.live.app.network.convert.JsonCallback
import ai.guiji.live.app.pojo.DigitalManInfo
import ai.guiji.live.app.pojo.SceneInfo
import ai.guiji.live.app.ui.adapter.SelectDigitalManAdapter
import android.annotation.SuppressLint
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import com.lzy.okgo.OkGo
import com.lzy.okgo.model.Response
import kotlinx.android.synthetic.main.activity_room_list.*
import kotlinx.android.synthetic.main.fragment_select_digital_man.*
import kotlinx.android.synthetic.main.fragment_select_digital_man.layout_empty
import org.json.JSONObject


class SelectDigitalManFragment : BaseFragment() {

    private var mRootView: View? = null
    private var adapter: SelectDigitalManAdapter? = null

    private var position = 0
    private val mList = ArrayList<DigitalManInfo>()

    private var mListener: SelectDigitalManFragmentListener? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // 初始化也获取下输入框中的内容
        position = arguments?.getInt("position") ?: 0
        mRootView = inflater.inflate(R.layout.fragment_select_digital_man, container, false)
        return mRootView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
    }

    @SuppressLint("NotifyDataSetChanged")
    fun refreshSelect(bean: DigitalManInfo) {
        for (item in mList) {
            item.select = item == bean
        }
        adapter?.notifyDataSetChanged()
    }

    private fun initView() {
        adapter =
            SelectDigitalManAdapter(mContext, object : SelectDigitalManAdapter.DigitalManListener {
                override fun onSelect(position: Int, item: DigitalManInfo) {
                    mListener?.onSelect(item)
                }
            })
        rv_digital.adapter = adapter

        if (position == 0) {
            queryMarket()
        } else {
            queryMy()
        }
    }

    private fun queryMarket() {
        mList.clear()
        val json = JSONObject()
        json.put("page", 1)
        json.put("size", 100)
        json.put("version", 0)
        json.put("orderBy", "")
        json.put("liveType", 0)

        json.put("groupId", "")
        json.put("name", "")
        json.put("address", "no_group_id")

        val filter = JSONObject()
        filter.put("applicationType", 2)
        json.put("sceneFilter", filter)
        json.put("sceneFilterFlag", true)

        Log.e("123", "queryMarket body: $json")
        OkGo.post<JSONObject>(
            String.format(
                Constant.API_LIST_MARKET,
                OkGo.getInstance().commonHeaders.get("token")
            )
        )
            .tag(TAG)
            .upJson(json.toString())
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    val body = response?.body()
                    Log.e("123", "listMarket body: $body")
                    val code = body?.optInt("code")
                    val msg = body?.optString("msg")
                    val data = body?.optJSONObject("data")
                    if (code == 200 && data != null) {
                        parseData(data)
                    } else {
                        Toast.makeText(mContext, "listLive msg: $msg", Toast.LENGTH_SHORT).show()
                        adapter?.setData(null)
                        layout_empty.visibility = View.VISIBLE
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    adapter?.setData(null)
                    layout_empty.visibility = View.VISIBLE
                }
            })

    }

    private fun queryMy() {
        mList.clear()
        val json = JSONObject()
        json.put("page", 1)
        json.put("size", 100)
        json.put("version", 0)
        json.put("orderBy", "")
        json.put("liveType", 0)

        json.put("groupId", "")
        json.put("name", "")
        json.put("address", "no_group_id")

        val filter = JSONObject()
        filter.put("applicationType", 2)
        json.put("sceneFilter", filter)
        json.put("sceneFilterFlag", true)

        OkGo.post<JSONObject>(
            String.format(
                Constant.API_LIST_PRIVATE_V2,
                OkGo.getInstance().commonHeaders.get("token")
            )
        )
            .tag(TAG)
            .upJson(json.toString())
            .execute(object : JsonCallback<JSONObject>() {
                override fun onSuccess(response: Response<JSONObject>?) {
                    val body = response?.body()
                    Log.e("123", "listPrivate body: $body")
                    val code = body?.optInt("code")
                    val msg = body?.optString("msg")
                    val data = body?.optJSONObject("data")
                    if (code == 200 && data != null) {
                        parseData(data)
                    } else {
                        Toast.makeText(mContext, "listLive msg: $msg", Toast.LENGTH_SHORT).show()
                        adapter?.setData(null)
                        layout_empty.visibility = View.VISIBLE
                    }
                }

                override fun onError(response: Response<JSONObject>?) {
                    super.onError(response)
                    adapter?.setData(null)
                    layout_empty.visibility = View.VISIBLE
                }
            })
    }

    private fun parseData(data: JSONObject) {
        val records = data.optJSONArray("records")
        records?.let {
            for (index in 0 until records.length()) {
                val info = DigitalManInfo()
                val item = records.getJSONObject(index)
                info.id = item.optInt("id")
                info.robotName = item.optString("robotName")
                info.robotCode = item.optString("robotCode")
                info.coverUrl = item.optString("coverUrl")
                info.delFlag = item.optInt("delFlag")

                val sceneList = ArrayList<SceneInfo>()
                val sceneListJson = item.optJSONArray("sceneList")
                sceneListJson?.let {
                    for (sceneIndex in 0 until sceneListJson.length()) {
                        val sceneObj = sceneListJson.getJSONObject(sceneIndex)
                        val scene = SceneInfo()
                        scene.id = sceneObj.optInt("id")
                        scene.sceneName = sceneObj.optString("sceneName")
                        scene.sceneCode = sceneObj.optString("sceneCode")
                        scene.sceneType = sceneObj.optInt("sceneType")
                        scene.coverUrl = sceneObj.optString("coverUrl")
                        scene.videoCoverUrl = sceneObj.optString("videoCoverUrl")
                        sceneList.add(scene)
                    }
                }
                info.sceneList = sceneList

                if (info.delFlag == 0 && info.sceneList.size > 0) {
                    mList.add(info)
                }
            }
        }
        adapter?.setData(mList)
        layout_empty.visibility = if (mList.size == 0) View.VISIBLE else View.INVISIBLE
    }

    fun setListener(listener: SelectDigitalManFragmentListener) {
        mListener = listener
    }

    interface SelectDigitalManFragmentListener {
        fun onSelect(item: DigitalManInfo)
    }

}